import lib.reg as reg
from lib.parsing.parser import Parser, ParsingError
from lib.checker import transform, dfa_transform, nfa_transform, check_task, check_alphabets
from enum import Enum
import enum


class Size(Enum):
    Empty = enum.auto()
    Finite = enum.auto()
    Infinite = enum.auto()
    Universal = enum.auto()


class Language():
    def __init__(self, string: str, task: str):
        self.string = string
        self.task = task
        self.dfa = None
        self.minimal = None
        self.size = None
        self.sizename = None
        self.example = None
        self.gen_lang_html()

    def gen_lang_html(self):
        parser = Parser()
        dfa = dfa_transform(self.string, self.task)
        self.minimal = parser.dfa_to_str(dfa.minimize().canonize())

        empty = dfa.is_empty()
        if empty:
            size = Size.Empty
        else:
            self.example = empty.counterexample
            if not empty.inf:
                size = Size.Finite
            else:
                size = Size.Infinite
                if dfa.is_universal():
                    size = Size.Universal

        name = {"Empty": "prázdný",
                "Finite": "konečný",
                "Infinite": "nekonečný",
                "Universal": "univerzální"}
        self.size = size
        self.sizename = name[size.name]
        self.dfa = dfa

    def gen_is_task(self, task: str) -> str:
        teacher = "NFA" if self.task == "EFA" else self.task
        parser = Parser()
        dfa = parser.dfa_to_str(self.dfa.canonize())
        return f"{teacher}-{task}-N:{dfa}"


class WebChecker:
    def __init__(self, student_string: str, task: str):
        self.student_string = student_string
        self.task = task
        self.task_solved = ""
        self.alphabets = ""
        self.eq = None
        self.teacher = None
        self.student = None
        self.is_task = None
        self.img_src = "disjunction"

    def dfa_task(self, teacher_type, teacher_string, task, student_string):
        try:
            teacher = Language(string=teacher_string, task=teacher_type)
            student = Language(string=student_string, task=self.task)
            student_solution = transform(student_string, task)

        except ParsingError as ex: # Parsing error, don't continue
            return(ex.args)

        self.teacher = teacher
        self.student = student
        self.is_task = teacher.gen_is_task(task=self.task)

        if isinstance(student_solution, reg.DFA) or isinstance(student_solution, reg.NFA):
            # "Odpověď splňuje požadovaný formalismus."
            self.task_solved = check_task(student_solution, task)

        if teacher.size == Size.Empty and student.size == Size.Empty:
            return True

        alphabets = check_alphabets(student_alpha=student.dfa.characters,
                                    teacher_alpha=teacher.dfa.characters, task=task)
        if alphabets != "":
            self.alphabets = alphabets
            return False

        self.eq = reg.DFA.is_equivalent(student.dfa, teacher.dfa)

        # self.img_src = self.picture(self.eq.inf is None)
        if self.task_solved == "" and self.eq.inf is None:
            return True
        return False


    def compare(self, teacher_string: str, teacher_type: str):
        #signal.alarm(50) #
        try:
            return self.dfa_task(teacher_type=teacher_type, teacher_string=teacher_string,
                                 task=self.task, student_string=self.student_string)

        except Exception as ex:
            print("Error inside of web checker:", ex.args)


    def convert(self, student_type):
        #signal.alarm(50)

        try:
            parser = Parser()

            if self.task in {"DFA", "TOT", "MIN"}:
                dfa = dfa_transform(self.student_string, student_type)

                if self.task == "DFA":
                    return parser.dfa_to_str(dfa)
                if self.task == "TOT":
                    return parser.dfa_to_str(dfa.total())
                if self.task == "MIN":
                    return parser.dfa_to_str(dfa.minimize().canonize())
                #if canonize: TODO checkbox
                #    return parser.dfa_to_str(automaton.canonize())

            nfa = nfa_transform(self.student_string, student_type)
            if self.task == "EFA":
                return parser.nfa_to_str(nfa)

            nfa = nfa.eliminate_epsilon()
            if self.task == "NFA":
                return parser.nfa_to_str(nfa)

            if self.task == "GRA":
                return parser.reggrammar_to_str(nfa.nfa_to_reggrammar().eliminate_useless())

        except ParsingError as ex:
            return("Chyba při parsování.")

        except Exception as ex:
            print("Error inside of web checker:", ex.args)

    def relation(self, eq: bool) -> str: # TODO test
        if not eq:
            if self.eq.left_counterexample is not None: # extra word
                if self.eq.left_counterexample is not None: # missing word
                    intersection = reg.DFA.intersection(self.student.dfa, self.teacher.dfa)
                    if intersection.is_empty(): # disjunction
                        return "disjunction"
                    else:
                        return "intersection"

                else: # teacher is subset of student
                    picture = "t_in_s"
            else: # student is subset of teacher
                picture = "s_in_t"

        else:
            picture = "eq"

        if self.teacher.size == Size.Empty or self.student.size == Size.Universal:
            picture += "_empty"
        elif self.teacher.size == Size.Universal or self.student.size == Size.Universal:
            picture += "_universal"

        return picture

